Package: MsBackendMetaboLights
Title: Retrieve Mass Spectrometry Data from MetaboLights
Version: 1.7.4
Authors@R: 
    c(person(given = "Johannes", family = "Rainer",
             email = "Johannes.Rainer@eurac.edu",
             role = c("aut", "cre"),
             comment = c(ORCID = "0000-0002-6977-7147")),
      person(given = "Philippine", family = "Louail",
             email = "philippine.louail@eurac.edu",
             role = c("aut"),
             comment = c(ORCID = "0009-0007-5429-6846")),
      person(given = "Gabriele", family = "Tomè",
             email = "gabriele.tome@eurac.edu",
             role = c("aut"),
             comment = c(ORCID = "0000-0002-3976-6068",
             fnd = "MetaRbolomics4Galaxy project (CUP: D53C25001030003) co-funded by the Autonomous Province of Bolzano under the Joint Projects South Tyrol–Germany 2025 program."))
	     )
Description: MetaboLights is one of the main public repositories for storage
             of metabolomics experiments, which includes analysis results as
	     well as raw data. The MsBackendMetaboLights package provides
	     functionality to retrieve and represent mass spectrometry (MS)
	     data from MetaboLights. Data files are downloaded and cached
	     locally avoiding repetitive downloads. MS data from metabolomics
	     experiments can thus be directly and seamlessly integrated into
	     R-based analysis workflows with the Spectra and
	     MsBackendMetaboLights package.
Depends: R (>= 4.2.0), Spectra (>= 1.15.12)
Imports: curl, ProtGenerics, BiocFileCache, S4Vectors, methods,
        progress, utils, MsCoreUtils (>= 1.23.8)
Suggests: testthat, rmarkdown, mzR, knitr, BiocStyle
License: Artistic-2.0
Encoding: UTF-8
VignetteBuilder: knitr
BugReports: https://github.com/RforMassSpectrometry/MsBackendMetaboLights/issues
URL: https://github.com/RforMassSpectrometry/MsBackendMetaboLights
biocViews: Infrastructure, MassSpectrometry, Metabolomics, DataImport,
        Proteomics
Roxygen: list(markdown=TRUE)
RoxygenNote: 8.0.0
git_url: https://git.bioconductor.org/packages/MsBackendMetaboLights
git_branch: devel
git_last_commit: c998a0d
git_last_commit_date: 2026-07-14
Repository: Bioconductor 3.24
Date/Publication: 2026-07-22
NeedsCompilation: no
Packaged: 2026-07-22 23:05:13 UTC; biocbuild
Author: Johannes Rainer [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-6977-7147>),
  Philippine Louail [aut] (ORCID:
    <https://orcid.org/0009-0007-5429-6846>),
  Gabriele Tomè [aut] (ORCID: <https://orcid.org/0000-0002-3976-6068>,
    fnd: MetaRbolomics4Galaxy project (CUP: D53C25001030003) co-funded
    by the Autonomous Province of Bolzano under the Joint Projects
    South Tyrol–Germany 2025 program.)
Maintainer: Johannes Rainer <Johannes.Rainer@eurac.edu>
