# Generated by roxygen2: do not edit by hand

export(MsBackendMetaboLights)
export(mtbls_assay_data)
export(mtbls_cached_data_files)
export(mtbls_delete_cache)
export(mtbls_ftp_path)
export(mtbls_list_files)
export(mtbls_metadata)
export(mtbls_sample_data)
export(mtbls_sync)
export(mtbls_sync_data_files)
exportClasses(MsBackendMetaboLights)
exportMethods(backendInitialize)
exportMethods(backendRequiredSpectraVariables)
importClassesFrom(Spectra,MsBackendDataFrame)
importClassesFrom(Spectra,MsBackendMzR)
importFrom(BiocFileCache,BiocFileCache)
importFrom(BiocFileCache,bfcinfo)
importFrom(BiocFileCache,bfcremove)
importFrom(BiocFileCache,bfcupdate)
importFrom(MsCoreUtils,retry)
importFrom(S4Vectors,DataFrame)
importFrom(Spectra,MsBackendMzR)
importFrom(curl,curl)
importFrom(curl,handle_setopt)
importFrom(curl,new_handle)
importFrom(methods,as)
importFrom(methods,callNextMethod)
importFrom(methods,new)
importFrom(methods,validObject)
importFrom(progress,progress_bar)
importFrom(utils,URLencode)
importFrom(utils,capture.output)
importFrom(utils,menu)
importFrom(utils,read.table)
importMethodsFrom(BiocFileCache,"bfcmeta<-")
importMethodsFrom(BiocFileCache,bfcmetalist)
importMethodsFrom(BiocFileCache,bfcquery)
importMethodsFrom(BiocFileCache,bfcrpath)
importMethodsFrom(ProtGenerics,backendInitialize)
importMethodsFrom(ProtGenerics,dataOrigin)
importMethodsFrom(Spectra,"[")
importMethodsFrom(Spectra,backendInitialize)
importMethodsFrom(Spectra,backendRequiredSpectraVariables)
